/*+==========================================================================
  File:      corhostdlg.cpp

  Summary:   The main dialog class for the corhost sample.
 
  Classes:   CCorHostDlg

  Functions: DisplayError
             
----------------------------------------------------------------------------
  This file is part of the Microsoft NGWS Samples.

  Copyright (C) 1998-1999 Microsoft Corporation.  All rights reserved.
==========================================================================+*/

#include "stdafx.h"
#include <stdio.h>
#include "CorHost.h"
#include "cor.h"
#include "mscoree.h"

#ifdef GetNextSibling
#undef GetNextSibling
#endif
#import <mscorlib.tlb> raw_interfaces_only high_property_prefixes("_get","_put","_putref")
using namespace ComRuntimeLibrary;

#include "CorHostDlg.h"
#include "EnumDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//
// Generic error display function ...
//
void DisplayError(HRESULT hr)
{
	CString m_errorText;
	static WORD wLangId = MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT);
	static DWORD dwFlags = FORMAT_MESSAGE_FROM_SYSTEM
		                 | FORMAT_MESSAGE_ALLOCATE_BUFFER
						 | FORMAT_MESSAGE_IGNORE_INSERTS;
	HLOCAL* pBuf = NULL;
	BOOL bOk = FormatMessage( dwFlags, NULL, hr, wLangId, (LPTSTR) &pBuf, 0, NULL );

	if( bOk )
	{	
		m_errorText = (PTCHAR)pBuf;
		LocalFree(pBuf);
	}
	else
	{
		AfxMessageBox(_T("Error Cannot be displayed"));
	}

	// Experience has shown that some, and perhaps all system error strings
	// have \r\n at the end. Remove it.
	static TCHAR szEndOfLine[] = _T("\r\n");
	int cchMsg = m_errorText.GetLength();
	if( cchMsg >= 2 && 
		_tcscmp( LPCTSTR(m_errorText) + cchMsg - 2, szEndOfLine ) == 0 )
	{
		*(m_errorText.GetBuffer(1) + cchMsg - 2) = _T('\0');
		m_errorText.ReleaseBuffer();
	}

	AfxMessageBox("FAILED hr: " + m_errorText);
}

/////////////////////////////////////////////////////////////////////////////
// CCorHostDlg dialog

CCorHostDlg::CCorHostDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCorHostDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCorHostDlg)
	m_currentAssemblyName = _T("");
	m_result = _T("");
	m_domainName = _T("Intial Domain");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_pCorHost = NULL;

	m_currentAssembly.m_name = _T("");
	m_currentAssembly.m_pAssembly = NULL;
	m_currentAssembly.m_pbFile = NULL;
}

CCorHostDlg::~CCorHostDlg()
{

}

void CCorHostDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCorHostDlg)
	DDX_Control(pDX, IDC_NEWDOMAIN, m_newDomain);
	DDX_Control(pDX, IDC_CLASSES, m_classes);
	DDX_Text(pDX, IDC_ASSEMBLIES, m_currentAssemblyName);
	DDX_Text(pDX, IDC_RESULT, m_result);
	DDX_Text(pDX, IDC_DOMAINNAME, m_domainName);
	DDX_Control(pDX, IDC_METHODS, m_methodsList);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCorHostDlg, CDialog)
	//{{AFX_MSG_MAP(CCorHostDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_EXECUTE, OnExecute)
	ON_BN_CLICKED(IDC_LISTCLASSES, OnListclasses)
	ON_BN_CLICKED(IDC_ENUMDOMAINS, OnEnumdomains)
	ON_BN_CLICKED(IDDONE, OnDone)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCorHostDlg message handlers

BOOL CCorHostDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	//
	// Start NGWS
	//
    HRESULT hr = CoCreateInstance(CLSID_CorRuntimeHost,NULL,CLSCTX_INPROC_SERVER,IID_ICorRuntimeHost,(void**)&m_pCorHost);
    if (FAILED(hr)) {
        TRACE(_T("Unable to create the cor runtime host"));
		DisplayError(hr);
    }
	
	if(FAILED(m_pCorHost->Start())) 
	{
        TRACE(_T("Unable to start NGWS"));
		DisplayError(hr);
		m_pCorHost->Release();
		m_pCorHost = NULL;
    }

	::CheckDlgButton(m_hWnd, IDC_ASSEMBLYNAME, BST_CHECKED);
	::SetFocus(::GetDlgItem(m_hWnd, IDC_ASSEMBLIES));
	return FALSE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CCorHostDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CCorHostDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CCorHostDlg::OnExecute() 
{	
	_Assembly *pAssembly = GetCurrentAssembly();

	if (!pAssembly)
	{
		CWnd::MessageBox(_T("You must first enter an assembly and choose a class to execute"));
		return;
	}

	//
	// Get the selected type from the list box and retrieve an instance of System.Type
	// from the assembly.
	//
	CString   typeName;
    _Type    *pType = NULL;
	HRESULT hr = S_OK;

	if (!m_classes.GetCount())
	{
		AfxMessageBox(_T("There are no classes to select"));
		return;
	}

	m_classes.GetLBText(m_classes.GetCurSel(), typeName);
	
    hr = pAssembly->GetType_2(_bstr_t(typeName), &pType);
    if(FAILED(hr)) {
        TRACE(_T("Could not find type\n"));
		DisplayError(hr);
		return;
    }
    
	//
	// Get the string representation of the type to display in the UI.  This at least 
	// shows the assembly was sucessfully loaded into the domain.
	//
	BSTR typeString;
	hr = pType->get_ToString(&typeString);

	if (FAILED(hr))
	{
		TRACE(_T("Could not create instance of the type\n"));
		DisplayError(hr);
		return;
	}

	//
	// Display the string in the ui
	//
	m_result = (TCHAR *)(_bstr_t(typeString, false));

	//
	// Get all the methods from the type
	//
	SAFEARRAY *psMethods;
	hr = pType->GetMethods_2(&psMethods);

	if (SUCCEEDED(hr))
	{
		long uBound, lBound, i = 0;
		IUnknown *pIUnknown = NULL;
		_MethodInfo *pMethodInfo = NULL; 

		::SafeArrayGetUBound(psMethods, 1, &uBound);
		::SafeArrayGetLBound(psMethods, 1, &lBound);

		m_methodsList.ResetContent();

		for (i = lBound; i <= uBound; i++)
		{
			::SafeArrayGetElement(psMethods, &i, &pIUnknown);

			HRESULT  hr = pIUnknown->QueryInterface(__uuidof(_MethodInfo), 
                                    (void**) &pMethodInfo);
			if(FAILED(hr)) 
			{
				TRACE(_T("Could not get the _MethodInfo interface\n"));
			}

			BSTR bstrSig;
			pMethodInfo->get_ToString(&bstrSig);

			m_methodsList.AddString((TCHAR *)(_bstr_t(bstrSig, false)));

			pMethodInfo->Release();
			pIUnknown->Release();
		}

	::SafeArrayDestroy(psMethods);
	}

	UpdateData(FALSE);

	pType->Release();
	
}

void CCorHostDlg::Cleanup()
{
	//
	// Clear out any existing "current" assembly
	//
	if (m_currentAssembly.m_pAssembly) m_currentAssembly.m_pAssembly->Release();
	if (m_currentAssembly.m_pbFile)    delete m_currentAssembly.m_pbFile;

	//
	// Free any domains we may have created.
	//
	vector<_AppDomain *>::iterator domainIter;
	_AppDomain *pDomain = NULL;

	for (domainIter = m_domains.begin(); domainIter != m_domains.end(); domainIter++)
    {
	  pDomain = *domainIter;
	  _ASSERT(pDomain);
	  pDomain->Release();
    }

	//
	// Release the interface used to control NGWS
	//
	if (m_pCorHost) 
	{
		m_pCorHost->Release();
	}
}

//
// Creates a new domain with the friendly name "name" and adds it to the
// list of domains
//
_AppDomain *CCorHostDlg::AddNewDomain(const CString& name)
{
	USES_CONVERSION;

	_ASSERT(m_pCorHost);

	IUnknown   *pAppDomainPunk = NULL;
	_AppDomain *pAppDomain = NULL;

	HRESULT hr = m_pCorHost->CreateDomain(T2OLE((LPCTSTR)name),
                                NULL,
                                NULL,
                                &pAppDomainPunk);

    if(FAILED(hr)) 
	{
        TRACE(_T("Could not create domain to load assembly into\n"));
		DisplayError(hr);
		return NULL;
    }
	else
	{
	    hr = pAppDomainPunk->QueryInterface(__uuidof(_AppDomain), 
                                        (void**) &pAppDomain);
		if(FAILED(hr)) 
		{
			TRACE(_T("Could not get the AppDomain interface\n"));
			DisplayError(hr);
			pAppDomainPunk->Release();
			return NULL;
		}
	}

	m_domains.push_back(pAppDomain);

	TRACE(_T("New Domain added\n"));

	BSTR bstrDomain = L"xxx";
	pAppDomain->get_BaseDirectory(&bstrDomain);

	return pAppDomain;

}

void CCorHostDlg::OnListclasses() 
{
	//
	// Verify we have a reasonable assembly name or path
	//
	UpdateData(TRUE);

	bool bIsPath = true;

	if (::IsDlgButtonChecked(m_hWnd, IDC_ASSEMBLYNAME) == BST_CHECKED)
	{
		bIsPath = false;
	}


	if (m_currentAssemblyName.IsEmpty())
	{
		CWnd::MessageBox(_T("Please enter an assembly name"));
		return;
	}

	//
	// Get a domain to load the assembly into.  If "new domain" was requested, or
	// the domain list is empty, create a new one, otherwise just reuse the first one
	// on the list
	//
	_AppDomain *pDomain = NULL;
	_Assembly  *pAssembly = NULL;

	if ((m_newDomain.GetCheck()) || (m_domains.size() == 0))
	{
		pDomain = AddNewDomain(m_domainName);
	}
	else
	{
		pDomain = m_domains[0];
	}

	_ASSERT(pDomain);

	//
	// Load the assembly into the domain
	//
	pAssembly = SetCurrentAssembly(m_currentAssemblyName, bIsPath, pDomain);

	if (!pAssembly)
	{
		// error raised in SetCurrentAssembly
		return;
	}
	
	//
	// Enumerate the types and populate the list box
	//
	m_classes.ResetContent();

	SAFEARRAY *psExportedTypes;
	HRESULT hr = pAssembly->GetExportedTypes(&psExportedTypes);

	if (FAILED(hr))
	{
		TRACE(_T("Unable to list the types in the assembly"));
		AfxMessageBox(_T("Unable to list the types in the assembly"));
		return;
	}

	long uBound, lBound, i = 0;
	IUnknown *pElement;
	
	::SafeArrayGetUBound(psExportedTypes, 1, &uBound);
	::SafeArrayGetLBound(psExportedTypes, 1, &lBound);

	BSTR typeName;	
	for (i = lBound; i <= uBound; i++)
	{
		_Type *pType = NULL;

		HRESULT hr = ::SafeArrayGetElement(psExportedTypes, &i, &pElement);

		if (SUCCEEDED(hr))
		{
			hr = pElement->QueryInterface(__uuidof(_Type), 
                                        (void**) &pType);
			if (SUCCEEDED(hr))
			{
				pType->get_FullName(&typeName);
				m_classes.AddString((TCHAR *)(_bstr_t(typeName, false)));
				pType->Release();
			}
			pElement->Release();

		}
		
	}

	::SafeArrayDestroy(psExportedTypes);

	m_classes.SelectString(0, (TCHAR *)(_bstr_t(typeName, false)));

}

//
// Loads the assembly identified by "name" into "pDomain".  We save information about this
// assembly so we can use it later when types are executed.
//
_Assembly *CCorHostDlg::SetCurrentAssembly(const CString& name, bool bIsPath, _AppDomain *pDomain)
{	
	HRESULT hr = S_OK;

	//
	// Clear out any existing "current" assembly
	//
	if (m_currentAssembly.m_pAssembly)
	{
		m_currentAssembly.m_pAssembly->Release();
		m_currentAssembly.m_pAssembly = NULL;
	}

	if (m_currentAssembly.m_pbFile)
	{
		delete m_currentAssembly.m_pbFile;
		m_currentAssembly.m_pbFile = NULL;
	}

	//
	// There are several versions of AppDomain::Load.  This method demonstrates two of them.
	// If the user specified as path to an assembly manifest, we load the entire contents of 
	// the file into memory and load the assembly from the memory stream.
	//
	// If the use just specified an assembly name, we just pass it on and ask
	// NGWS to find the assembly and load it from disk.
	//
	// Note the distinction here between "assembly path" and "assembly name" and how I've decided
	// which version of LoadAssembly to use, is arbitrary.  You could load an assembly from disk
	// given a path, for example.
	//
	// This just seemed like a convenient way to demonstrate a few different ways to do this.
	//
	if (bIsPath)
	{
		//
		// Load the assembly from a path into memory ...
		//
		HANDLE  hFile = 0;
		DWORD   cbFile = 0;

		if ((hFile = ::CreateFile(name,
								GENERIC_READ,
								FILE_SHARE_READ,
								NULL,                   // lpsa
								OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL,
								NULL)) == INVALID_HANDLE_VALUE) 
		{
			CWnd::MessageBox(_T("Could not open Assembly file\n"));
			return NULL;
		}
    
		if((cbFile = GetFileSize(hFile, NULL)) == 0xffffffff) {
			CWnd::MessageBox(_T("Could not get the size of Assembly file\n"));
			return NULL;
		}
    
		m_currentAssembly.m_pbFile = new BYTE[cbFile]; 

		_ASSERT (hFile);
		_ASSERT (m_currentAssembly.m_pbFile);
    
		// Read the encoding message
		DWORD dwBytesRead;
		if(!ReadFile(hFile, 
					 m_currentAssembly.m_pbFile,
					 cbFile,
					 &dwBytesRead,
					 NULL) ||
		   dwBytesRead != cbFile) 
		{
			TRACE(_T("Could not read all of Assembly1.dll\n"));

			// clean up and get out
			::CloseHandle(hFile);
			return NULL;
		}

		TRACE(_T("Assembly file read successfully\n"));

		SAFEARRAY spbFile;
		ZeroMemory(&spbFile, sizeof(SAFEARRAY));
		spbFile.cDims = 1;
		spbFile.cbElements = 1;
		spbFile.fFeatures = FADF_FIXEDSIZE | FADF_STATIC;
		spbFile.pvData = m_currentAssembly.m_pbFile;
		spbFile.rgsabound[0].cElements = cbFile;
		spbFile.rgsabound[1].lLbound = 0;


		//
		// This version of LoadAssembly takes a pointer to memory
		//
		hr = pDomain->Load_5(&spbFile, &(m_currentAssembly.m_pAssembly));

		if(FAILED(hr)) {
		   TRACE(_T("Could not load the assembly into the appdomain"));
		   DisplayError(hr);

			// clean up and get out
			::CloseHandle(hFile);
			return NULL;
		}

		TRACE(_T("Assembly loaded!\n"));

		::CloseHandle(hFile);
	}
	else
	{
		
		hr = pDomain->Load_4(_bstr_t(name), &(m_currentAssembly.m_pAssembly));

		if(FAILED(hr)) {
			TRACE(_T("Could not load the assembly into the appdomain"));
			DisplayError(hr);
			return NULL;
		}

		TRACE(_T("Assembly loaded!\n"));

	}

	return m_currentAssembly.m_pAssembly;
}

_Assembly  *CCorHostDlg::GetCurrentAssembly()
{
	return m_currentAssembly.m_pAssembly;
}

//
// Displays a dialog showing the friendly names of all the domain we've created.
//
void CCorHostDlg::OnEnumdomains() 
{
	CEnumDialog dlgEnum;

	dlgEnum.SetHost(m_pCorHost);
	dlgEnum.DoModal();
	
}

void CCorHostDlg::OnDone() 
{
	Cleanup();
	CDialog::OnOK();
}

void CCorHostDlg::OnOK()
{
	return;
}