/*+==========================================================================
  File:      enumdialog.cpp

  Summary:   A utility dialog that displays the names of all the App domains in a process
 
  Classes:   CEnumDialog

  Functions: None
             
----------------------------------------------------------------------------
  This file is part of the Microsoft NGWS Samples.

  Copyright (C) 1998-1999 Microsoft Corporation.  All rights reserved.
==========================================================================+*/

#include "stdafx.h"
#include "CorHost.h"
#include "cor.h"
#include "mscoree.h"

#ifdef GetNextSibling
#undef GetNextSibling
#endif
#import <mscorlib.tlb> raw_interfaces_only high_property_prefixes("_get","_put","_putref")
using namespace ComRuntimeLibrary;

#include "EnumDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEnumDialog dialog


CEnumDialog::CEnumDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CEnumDialog::IDD, pParent), m_pCorHost(NULL)
{
	//{{AFX_DATA_INIT(CEnumDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CEnumDialog::~CEnumDialog()
{
	if (m_pCorHost)
	{
		m_pCorHost->Release();
	}
}


void CEnumDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEnumDialog)
	DDX_Control(pDX, IDC_DOMAINLIST, m_domainList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEnumDialog, CDialog)
	//{{AFX_MSG_MAP(CEnumDialog)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEnumDialog message handlers
BOOL CEnumDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	_ASSERT(m_pCorHost);

	//
	// Enumerate through all the domains, get their friendly names, and populate 
	// the list box
	//
	HDOMAINENUM hEnum;
	HRESULT     hr = S_OK;

	hr = m_pCorHost->EnumDomains(&hEnum);

	if (FAILED(hr))
	{
		CWnd::MessageBox(_T("Can't get domain enumerator"));
		return TRUE;
	}

	IUnknown *pDomainPunk = NULL;
	while (m_pCorHost->NextDomain(hEnum, &pDomainPunk) == S_OK)
	{
		_AppDomain *pDomain = NULL;

		hr = pDomainPunk->QueryInterface(__uuidof(_AppDomain), 
                                        (void**) &pDomain);
		if(FAILED(hr)) 
		{
			CWnd::MessageBox(_T("Failure while enumerating domains"));
			pDomainPunk->Release();
			return NULL;
		}

		BSTR bstrDomain = L"xxx";
		hr = pDomain->get_FriendlyName(&bstrDomain);

		if (FAILED(hr))
		{
			CWnd::MessageBox(_T("Failure while enumerating domains"));
		}
		else
		{
			if (bstrDomain)
			{
				m_domainList.AddString((TCHAR *)(_bstr_t(bstrDomain, false)));
			}
			else
			{
				m_domainList.AddString(_T("Domain has no name"));
			}
		}

		pDomain->Release();
		pDomainPunk->Release();

	}
	m_pCorHost->CloseEnum(hEnum);



	return TRUE;
}

void CEnumDialog::SetHost(ICorRuntimeHost *pCorHost)
{
	_ASSERT(pCorHost);

	m_pCorHost = pCorHost;
	pCorHost->AddRef();
}
